@@ -3,42 +3,31 @@ Create a NeXus file for time-resolved collections on I19-2.
 Available detectors: Tristan 10M, Eiger 2X 4M.
 """
 
-import sys
-
-import h5py
 import glob
 import logging
-
-from pathlib import Path
-
+import sys
 from collections import namedtuple
 from datetime import datetime
-from typing import Union, Tuple
+from pathlib import Path
+from typing import Tuple, Union
 
-from .I19_2_params import (
-    source,
-    goniometer_axes,
-    tristan10M_params,
-    eiger4M_params,
-    dset_links,
-)
-
-from .. import (
-    get_iso_timestamp,
-    get_nexus_filename,
-)
-
-from ..nxs_write import (
-    calculate_rotation_scan_range,
-)
+import h5py
 
+from .. import get_iso_timestamp, get_nexus_filename
+from ..nxs_write import calculate_rotation_scan_range
 from ..nxs_write.NexusWriter import call_writers
 from ..nxs_write.NXclassWriters import write_NXdatetime, write_NXentry
-
 from ..tools.ExtendedRequest import ExtendedRequestIO
 from ..tools.GDAjson2params import (
-    read_geometry_from_json,
     read_detector_params_from_json,
+    read_geometry_from_json,
+)
+from .I19_2_params import (
+    dset_links,
+    eiger4M_params,
+    goniometer_axes,
+    source,
+    tristan10M_params,
 )
 
 # Define a logger object and a formatter
@@ -149,9 +138,8 @@ def tristan_writer(
     detector["flatfield"] = flatfieldfile
     # If these two could instead be passed, I'd be happier...
 
-    # Define SCANS dictionary
-    SCANS = {}
-    SCANS["rotation"] = {scan_axis: scan_range}
+    # Define OSC scans dictionary
+    OSC = {scan_axis: scan_range}
 
     # Get on with the writing now...
     try:
@@ -166,7 +154,6 @@ def tristan_writer(
                 nxsfile,
                 [TR.meta_file],
                 coordinate_frame,
-                SCANS,
                 (detector["mode"], None),
                 goniometer,
                 detector,
@@ -174,6 +161,7 @@ def tristan_writer(
                 source,
                 beam,
                 attenuator,
+                OSC,
             )
 
             # write_NXdatetime(nxsfile, (None, timestamps[1]))
@@ -225,9 +213,8 @@ def eiger_writer(
         n_images=n_frames,
     )
 
-    # Define SCANS dictionary
-    SCANS = {}
-    SCANS["rotation"] = {scan_axis: scan_range}
+    # Define OSC scans dictionary
+    OSC = {scan_axis: scan_range}
 
     # Get on with the writing now...
     try:
@@ -241,7 +228,6 @@ def eiger_writer(
                 nxsfile,
                 filenames,
                 coordinate_frame,
-                SCANS,
                 (detector["mode"], n_frames),
                 goniometer,
                 detector,
@@ -249,6 +235,8 @@ def eiger_writer(
                 source,
                 beam,
                 attenuator,
+                OSC,
+                transl_scan=None,
                 metafile=TR.meta_file,
                 link_list=dset_links,
             )
@@ -400,9 +388,10 @@ def write_nxs(**tr_params):
 
 
 def main():
-    " Call from the beamline"
+    "Call from the beamline"
     # Not the best but it should do the job
     import argparse
+
     from ..command_line import version_parser
 
     parser = argparse.ArgumentParser(description=__doc__, parents=[version_parser])
