@@ -2,33 +2,39 @@
 Command line tool to generate NeXus files.
 """
 
-import sys
-import glob
-import h5py
-import time
-import logging
 import argparse
-import freephil
+import glob
+import logging
+import sys
+import time
+from datetime import datetime
+from pathlib import Path
 
+import freephil
+import h5py
 import numpy as np
 
-from pathlib import Path
-from datetime import datetime
-
-from . import (
-    version_parser,
-    detectormode_parser,
-    nexus_parser,
-    demo_parser,
-    add_tristan_spec,
-)
 from .. import (
-    get_nexus_filename,
     get_filename_template,
     get_iso_timestamp,
+    get_nexus_filename,
+    units_of_time,
+)
+from ..nxs_write.NexusWriter import (  # write_nexus_demo, write_nexus
+    ScanReader,
+    call_writers,
+)
+from ..nxs_write.NXclassWriters import write_NXdatetime, write_NXentry, write_NXnote
+from ..tools.DataWriter import generate_event_files, generate_image_files
+from ..tools.MetaReader import overwrite_beam, overwrite_detector
+from ..tools.VDS_tools import image_vds_writer, vds_file_writer
+from . import (
+    add_tristan_spec,
+    demo_parser,
+    detectormode_parser,
+    nexus_parser,
+    version_parser,
 )
-from ..nxs_write.NexusWriter import write_nexus, write_nexus_demo
-from ..nxs_write.NXclassWriters import write_NXnote
 
 # Define a logger object and a formatter
 logger = logging.getLogger("NeXusGenerator")
@@ -39,6 +45,11 @@ formatter = logging.Formatter("%(asctime)s %(name)s %(levelname)s %(message)s")
 # Phil scopes
 master_phil = freephil.parse(
     """
+    output {
+      master_filename = None
+        .type = path
+        .help = "Filename for master file"
+    }
     input {
       datafile = None
         .multiple = True
@@ -50,6 +61,9 @@ master_phil = freephil.parse(
       vds_writer = *None dataset file
         .type = choice
         .help = "If not None, write vds along with external link to data in NeXus file, or create _vds.h5 file."
+      snaked = False
+        .type = bool
+        .help = "Grid scan parameter. If True, the writer will draw a snaked grid."
     }
 
     include scope nexgen.command_line.nxs_phil.goniometer_scope
@@ -80,6 +94,9 @@ demo_phil = freephil.parse(
       vds_writer = *None dataset file
         .type = choice
         .help = "If not None, either write a vds in the nexus file or create also a _vds.h5 file."
+      snaked = False
+        .type = bool
+        .help = "Grid scan parameter. If True, the writer will draw a snaked grid."
     }
 
     include scope nexgen.command_line.nxs_phil.goniometer_scope
@@ -95,6 +112,11 @@ demo_phil = freephil.parse(
 
 meta_phil = freephil.parse(
     """
+    output {
+      master_filename = None
+        .type = path
+        .help = "Filename for master file"
+    }
     input {
       metafile = None
         .type = path
@@ -109,6 +131,9 @@ meta_phil = freephil.parse(
       vds_writer = *None dataset file
         .type = choice
         .help = "If not None, write vds along with external link to data in NeXus file, or create _vds.h5 file."
+      snaked = False
+        .type = bool
+        .help = "Grid scan parameter. If True, the writer will draw a snaked grid."
     }
 
     include scope nexgen.command_line.nxs_phil.goniometer_scope
@@ -120,8 +145,6 @@ meta_phil = freephil.parse(
     include scope nexgen.command_line.nxs_phil.module_scope
 
     include scope nexgen.command_line.nxs_phil.timestamp_scope
-
-    # include scope nexgen.command_line.nexus_generator.master_phil
     """,
     process_includes=True,
 )
@@ -140,6 +163,15 @@ parser.add_argument(
     dest="show_config",
     help="Show the configuration parameters.",
 )
+parser.add_argument(
+    "-a",
+    "--attributes-level",
+    default=0,
+    type=int,
+    dest="attributes_level",
+    help="Set the attributes level for showing the configuration parameters.",
+)
+
 
 # CLIs
 def write_NXmx_cli(args):
@@ -147,14 +179,21 @@ def write_NXmx_cli(args):
     working_phil = master_phil.fetch(cl.process_and_fetch(args.phil_args))
     params = working_phil.extract()
 
+    if args.show_config:
+        working_phil.show(attributes_level=args.attributes_level)
+        sys.exit()
+
     # Path to data file
     datafiles = [Path(d).expanduser().resolve() for d in params.input.datafile]
 
     # Get NeXus file name
-    master_file = get_nexus_filename(datafiles[0])
+    if params.output.master_filename:
+        master_file = Path(params.output.master_filename).expanduser().resolve()
+    else:
+        master_file = get_nexus_filename(datafiles[0])
 
     # Start logger
-    logfile = datafiles[0].parent / "generate_nexus.log"
+    logfile = master_file.parent / "generate_nexus.log"
     # Define a file handler for logging
     FH = logging.FileHandler(logfile, mode="a")
     FH.setLevel(logging.DEBUG)
@@ -186,6 +225,22 @@ def write_NXmx_cli(args):
     # If dealing with a tristan detector, add its specifications to detector scope.
     if "TRISTAN" in detector.description.upper():
         add_tristan_spec(detector, params.tristanSpec)
+        logger.info("Tristan specs added to params for writing.")
+
+    # Define images vs events based on detector mode
+    logger.info(f"Data type: {detector.mode}.")
+    if detector.mode == "events":
+        data_type = ("events", None)
+    else:
+        if len(datafiles) == 1:
+            with h5py.File(datafiles[0], "r") as f:
+                num_images = f["data"].shape[0]
+        else:
+            from ..nxs_write import find_number_of_images
+
+            num_images = find_number_of_images(datafiles)
+        data_type = ("images", num_images)
+        logger.info(f"Total number of images: {num_images}")
 
     # Log information
     logger.info("Source information")
@@ -226,7 +281,25 @@ def write_NXmx_cli(args):
             f"Goniometer axis: {axes[j]} => {vector} ({goniometer.types[j]}) on {goniometer.depends[j]}. {goniometer.starts[j]} {goniometer.ends[j]} {goniometer.increments[j]}"
         )
 
-    logger.info("")
+    logger.info("\n")
+
+    # Define rotation and translation axes
+    OSC, TRANSL = ScanReader(
+        goniometer.__dict__,
+        data_type[0],
+        n_images=data_type[1],
+        snaked=params.input.snaked,
+    )
+    # Log scan information
+    logger.info(f"Rotation scan axis: {list(OSC.keys())[0]}.")
+    logger.info(
+        f"Scan from {list(OSC.values())[0][0]} to {list(OSC.values())[0][-1]}.\n"
+    )
+    if TRANSL:
+        logger.info(f"Scan along the {list(TRANSL.keys())} axes.")
+        for k, v in TRANSL.items():
+            logger.info(f"{k} scan from {v[0]} to {v[-1]}.")
+    logger.info("\n")
 
     logger.info(
         f"Detector information:\n {detector.description}, {detector.detector_type}"
@@ -273,24 +346,27 @@ def write_NXmx_cli(args):
     logger.info(f"Fast axis at datum position: {module.fast_axis}")
     logger.info(f"Slow_axis at datum position: {module.slow_axis}")
     if module.module_offset == "0":
-        logger.warning(f"module_offset field will not be written.")
+        logger.warning("module_offset field will not be written.")
     logger.info("")
 
     logger.info("Start writing NeXus file ...")
     try:
         with h5py.File(master_file, "x") as nxsfile:
-            write_nexus(
+            write_NXentry(nxsfile)
+
+            call_writers(
                 nxsfile,
                 datafiles,
-                goniometer,
-                detector,
-                module,
-                source,
-                beam,
-                attenuator,
-                timestamps,
                 cf,
-                params.input.vds_writer,
+                data_type,
+                goniometer.__dict__,
+                detector.__dict__,
+                module.__dict__,
+                source.__dict__,
+                beam.__dict__,
+                attenuator.__dict__,
+                OSC,
+                TRANSL,
             )
 
             # Check and save pump status
@@ -304,6 +380,23 @@ def write_NXmx_cli(args):
                 }
                 write_NXnote(nxsfile, "/entry/source/notes", pump_info)
 
+            # Write VDS
+            if data_type[0] == "images" and params.input.vds_writer == "dataset":
+                logger.info("Calling VDS writer ...")
+                image_vds_writer(nxsfile, (data_type[1], *detector.image_size))
+            elif data_type[0] == "images" and params.input.vds_writer == "file":
+                logger.info(
+                    "Calling VDS writer to write a Virtual Dataset file and relative link."
+                )
+                vds_file_writer(
+                    nxsfile, datafiles, (data_type[1], *detector.image_size)
+                )
+            else:
+                logger.info("VDS won't be written.")
+
+            # Write /entry/start_time and /entry/end_time
+            write_NXdatetime(nxsfile, timestamps)
+
         logger.info(f"{master_file} correctly written.")
     except Exception as err:
         logger.info(
@@ -320,6 +413,10 @@ def write_demo_cli(args):
     working_phil = demo_phil.fetch(cl.process_and_fetch(args.phil_args))
     params = working_phil.extract()
 
+    if args.show_config:
+        working_phil.show(attributes_level=args.attributes_level)
+        sys.exit()
+
     # Path to file
     master_file = Path(params.output.master_filename).expanduser().resolve()
     # Just in case ...
@@ -395,6 +492,31 @@ def write_demo_cli(args):
 
     logger.info("")
 
+    # Define rotation and translation axes
+    OSC, TRANSL = ScanReader(
+        goniometer.__dict__,
+        data_type[0],
+        n_images=data_type[1],
+        snaked=params.input.snaked,
+    )
+    print(OSC)
+    # Log scan information
+    logger.info(f"Rotation scan axis: {list(OSC.keys())[0]}.")
+    logger.info(
+        f"Scan from {list(OSC.values())[0][0]} to {list(OSC.values())[0][-1]}.\n"
+    )
+    if TRANSL:
+        logger.info(f"Scan along the {list(TRANSL.keys())} axes.")
+        for k, v in TRANSL.items():
+            logger.info(f"{k} scan from {v[0]} to {v[-1]}.")
+    logger.info("\n")
+
+    # Fix the number of images if not passed from command line.
+    if data_type[0] == "images" and data_type[1] is None:
+        data_type = ("images", len(list(OSC.values())[0]))
+        logger.warning(f"Total number of images updated to: {data_type[1]}")
+        logger.warning("\n")
+
     logger.info("Detector information:\n%s" % detector.description)
     logger.info(
         f"Sensor made of {detector.sensor_material} x {detector.sensor_thickness}"
@@ -436,26 +558,79 @@ def write_demo_cli(args):
     logger.info(f"Slow_axis at datum position: {module.slow_axis}")
     logger.info("")
 
+    # Figure out how many files will need to be written
+    logger.info("Calculating number of files to write ...")
+    if data_type[0] == "events":
+        # Determine the number of files. Write one file per module.
+        # FIXME Either a 10M or a 2M, no other possibilities at this moment.
+        n_files = 10 if "10M" in detector.description.upper() else 2
+    else:
+        # The maximum number of images being written each dataset is 1000
+        if data_type[1] <= 1000:
+            n_files = 1
+        else:
+            n_files = int(np.ceil(data_type[1] / 1000))
+
+    logger.info("%d file(s) containing blank data to be written." % n_files)
+
+    # Get datafile list
+    datafiles = [
+        Path(data_file_template % (n + 1)).expanduser().resolve()
+        for n in range(n_files)
+    ]
+
+    logger.info("Calling data writer ...")
+    # Write data files
+    if data_type[0] == "images":
+        generate_image_files(
+            datafiles, detector.image_size, detector.description, data_type[1]
+        )
+    else:
+        exp_time = units_of_time(detector.exposure_time)
+        generate_event_files(
+            datafiles, data_type[1], detector.description, exp_time.magnitude
+        )
+
+    logger.info("\n")
+
     # Record string with start_time
     start_time = datetime.fromtimestamp(time.time()).strftime("%A, %d. %B %Y %I:%M%p")
 
-    logger.info("Start writing NeXus and data files ...")
+    logger.info("Start writing demo NXmx NeXus and data files ...")
     try:
         with h5py.File(master_file, "x") as nxsfile:
-            write_nexus_demo(
+            write_NXentry(nxsfile)
+            call_writers(
                 nxsfile,
-                data_file_template,
-                data_type,
+                datafiles,
                 cf,
-                goniometer,
-                detector,
-                module,
-                source,
-                beam,
-                attenuator,
-                params.input.vds_writer,
+                data_type,
+                goniometer.__dict__,
+                detector.__dict__,
+                module.__dict__,
+                source.__dict__,
+                beam.__dict__,
+                attenuator.__dict__,
+                OSC,
+                TRANSL,
             )
 
+            # Write VDS
+            if data_type[0] == "images" and params.input.vds_writer == "dataset":
+                logger.info(
+                    "Calling VDS writer to write a Virtual Dataset under /entry/data/data"
+                )
+                image_vds_writer(nxsfile, (data_type[1], *detector.image_size))
+            elif data_type[0] == "images" and params.input.vds_writer == "file":
+                logger.info(
+                    "Calling VDS writer to write a Virtual Dataset file and relative link."
+                )
+                vds_file_writer(
+                    nxsfile, datafiles, (data_type[1], *detector.image_size)
+                )
+            else:
+                logger.info("VDS won't be written.")
+
             # Check and save pump status
             if params.pump_probe.pump_status is True:
                 logger.info(
@@ -473,8 +648,11 @@ def write_demo_cli(args):
             )
 
             # Write /entry/start_time and /entry/end_time
-            nxsfile.create_dataset("/entry/start_time", data=np.string_(start_time))
-            nxsfile.create_dataset("/entry/end_time", data=np.string_(end_time))
+            timestamps = (get_iso_timestamp(start_time), get_iso_timestamp(end_time))
+            logger.info("Writing timestamps to NeXus.")
+            logger.info(f"Start time: {timestamps[0]}")
+            logger.info(f"End time: {timestamps[1]}")
+            write_NXdatetime(nxsfile, timestamps)
         logger.info(f"{master_file} correctly written.")
     except Exception as err:
         logger.info(
@@ -490,6 +668,10 @@ def write_with_meta_cli(args):
     working_phil = meta_phil.fetch(cl.process_and_fetch(args.phil_args))
     params = working_phil.extract()
 
+    if args.show_config:
+        working_phil.show(attributes_level=args.attributes_level)
+        sys.exit()
+
     # Path to meta file
     if params.input.metafile:
         metafile = Path(params.input.metafile).expanduser().resolve()
@@ -499,7 +681,10 @@ def write_with_meta_cli(args):
         )
 
     # Get NeXus filename
-    master_file = get_nexus_filename(metafile)
+    if params.output.master_filename:
+        master_file = Path(params.output.master_filename).expanduser().resolve()
+    else:
+        master_file = get_nexus_filename(metafile)
 
     # If no datafile has been passed, look for them in the directory
     if params.input.datafile:
@@ -513,7 +698,7 @@ def write_with_meta_cli(args):
         ]
 
     # Start logger
-    logfile = metafile.parent / "generate_nexus_from_meta.log"
+    logfile = master_file.parent / "generate_nexus_from_meta.log"
     # Define a file handler for logging
     FH = logging.FileHandler(logfile, mode="a")
     FH.setLevel(logging.DEBUG)
@@ -547,6 +732,21 @@ def write_with_meta_cli(args):
     if "TRISTAN" in detector.description.upper():
         add_tristan_spec(detector, params.tristanSpec)
 
+    # Define images vs events based on detector mode
+    logger.info(f"Data type: {detector.mode}.")
+    if detector.mode == "events":
+        data_type = ("events", None)
+    else:
+        if len(datafiles) == 1:
+            with h5py.File(datafiles[0], "r") as f:
+                num_images = f["data"].shape[0]
+        else:
+            from ..nxs_write import find_number_of_images
+
+            num_images = find_number_of_images(datafiles)
+        data_type = ("images", num_images)
+        logger.info(f"Total number of images: {num_images}")
+
     # Log information
     logger.info("Source information")
     logger.info(f"Facility: {source.name} - {source.type}.")
@@ -586,7 +786,25 @@ def write_with_meta_cli(args):
             f"Goniometer axis: {axes[j]} => {vector} ({goniometer.types[j]}) on {goniometer.depends[j]}. {goniometer.starts[j]} {goniometer.ends[j]} {goniometer.increments[j]}"
         )
 
-    logger.info("")
+    logger.info("\n")
+
+    # Define rotation and translation axes
+    OSC, TRANSL = ScanReader(
+        goniometer.__dict__,
+        data_type[0],
+        n_images=data_type[1],
+        snaked=params.input.snaked,
+    )
+    # Log scan information
+    logger.info(f"Rotation scan axis: {list(OSC.keys())[0]}.")
+    logger.info(
+        f"Scan from {list(OSC.values())[0][0]} to {list(OSC.values())[0][-1]}.\n"
+    )
+    if TRANSL:
+        logger.info(f"Scan along the {list(TRANSL.keys())} axes.")
+        for k, v in TRANSL.items():
+            logger.info(f"{k} scan from {v[0]} to {v[-1]}.")
+    logger.info("\n")
 
     if detector.description is None:
         logger.warning("No detector description provided, exit.")
@@ -637,30 +855,42 @@ def write_with_meta_cli(args):
     logger.info(f"Fast axis at datum position: {module.fast_axis}")
     logger.info(f"Slow_axis at datum position: {module.slow_axis}")
     if module.module_offset == "0":
-        logger.warning(f"module_offset field will not be written.")
+        logger.warning("module_offset field will not be written.")
     logger.info("")
 
     if args.no_ow:
         logger.warning(f"The following datasets will not be overwritten: {args.no_ow}")
-        metainfo = (metafile, args.no_ow)
+        ignore = args.no_ow
     else:
-        metainfo = (metafile, None)
+        ignore = None
+
+    logger.info("Looking through _meta.h5 file for metadata.")
+    # overwrite detector, overwrite beam, get list of links for nxdetector and nxcollection
+
+    with h5py.File(metafile, "r") as mf:
+        overwrite_beam(mf, detector.description, beam)
+        link_list = overwrite_detector(mf, detector, ignore)
+
     logger.info("Start writing NeXus file ...")
     try:
         with h5py.File(master_file, "x") as nxsfile:
-            write_nexus(
+            write_NXentry(nxsfile)
+
+            call_writers(
                 nxsfile,
                 datafiles,
-                goniometer,
-                detector,
-                module,
-                source,
-                beam,
-                attenuator,
-                timestamps,
                 cf,
-                params.input.vds_writer,
-                metainfo,
+                data_type,
+                goniometer.__dict__,
+                detector.__dict__,
+                module.__dict__,
+                source.__dict__,
+                beam.__dict__,
+                attenuator.__dict__,
+                OSC,
+                TRANSL,
+                metafile,
+                link_list,
             )
 
             # Check and save pump status
@@ -674,6 +904,23 @@ def write_with_meta_cli(args):
                 }
                 write_NXnote(nxsfile, "/entry/source/notes", pump_info)
 
+            # Write VDS
+            if data_type[0] == "images" and params.input.vds_writer == "dataset":
+                logger.info("Calling VDS writer ...")
+                image_vds_writer(nxsfile, (data_type[1], *detector.image_size))
+            elif data_type[0] == "images" and params.input.vds_writer == "file":
+                logger.info(
+                    "Calling VDS writer to write a Virtual Dataset file and relative link."
+                )
+                vds_file_writer(
+                    nxsfile, datafiles, (data_type[1], *detector.image_size)
+                )
+            else:
+                logger.info("VDS won't be written.")
+
+            # Write /entry/start_time and /entry/end_time
+            write_NXdatetime(nxsfile, timestamps)
+
             logger.info(f"{master_file} correctly written.")
     except Exception as err:
         logger.info(
